//+------------------------------------------------------------------+
//|                                       AdaptiveLaguerre_v1_SW.mq4 |
//|                                Copyright  2008, TrendLaboratory |
//|            http://finance.groups.yahoo.com/group/TrendLaboratory |
//|                                   E-mail: igorad2003@yahoo.co.uk |
//+------------------------------------------------------------------+
// List of Prices:
// Price    = 0 - Close  
// Price    = 1 - Open  
// Price    = 2 - High  
// Price    = 3 - Low  
// Price    = 4 - Median Price   = (High+Low)/2  
// Price    = 5 - Typical Price  = (High+Low+Close)/3  
// Price    = 6 - Weighted Close = (High+Low+Close*2)/4
// Price    = 7 - Heiken Ashi Close  
// Price    = 8 - Heiken Ashi Open
// Price    = 9 - Heiken Ashi High
// Price    =10 - Heiken Ashi Low

// List of MAs for Gamma Smoothing:
// MA_Method= 0: SMA        - Simple Moving Average
// MA_Method= 1: EMA        - Exponential Moving Average
// MA_Method= 2: Wilder     - Wilder Exponential Moving Average
// MA_Method= 3: LWMA       - Linear Weighted Moving Average 
// MA_Method= 4: SineWMA    - Sine Weighted Moving Average
// MA_Method= 5: TriMA      - Triangular Moving Average
// MA_Method= 6: LSMA       - Least Square Moving Average (or EPMA, Linear Regression Line)
// MA_Method= 7: SMMA       - Smoothed Moving Average
// MA_Method= 8: HMA        - Hull Moving Average by Alan Hull
// MA_Method= 9: ZeroLagEMA - Zero-Lag Exponential Moving Average
// MA_Method=10: DEMA       - Double Exponential Moving Average by Patrick Mulloy
// MA_Method=11: T3         - T3 by T.Tillson
// MA_Method=12: ITrend     - Instantaneous Trendline by J.Ehlers
// MA_Method=13: Median     - Moving Median
// MA_Method=14: GeoMean    - Geometric Mean
// MA_Method=15: REMA       - Regularized EMA by Chris Satchwell
// MA_Method=16: ILRS       - Integral of Linear Regression Slope 
// MA_Method=17: IE/2       - Combination of LSMA and ILRS 
// MA_Method=18: TriMAgen   - Triangular Moving Average generalized by J.Ehlers

 
#property copyright "Copyright  2008, TrendLaboratory"
#property link      "http://finance.groups.yahoo.com/group/TrendLaboratory"

#property indicator_separate_window
#property indicator_buffers 1
#property indicator_color1  Gold
#property indicator_width1  2  
//---- 
extern int     TimeFrame         =   0;   
extern int     Price             =   0;
extern int     L_Period          =   4;
extern int     AdaptLength       =  20;
extern double  Gamma             = 0.8;
extern int     GammaMode         =   2;
extern int     GammaSmooth       =   5;
extern int     GammaSmoothMode   =   5;
extern int     L_Shift           =   0;

//---- 
double Laguerre[];
//----
double tmp[][6];
double L[][10];
double haClose[], haOpen[], haHigh[], haLow[];
int    draw_begin, pBars, mcnt_bars; 
string short_name;
datetime pTime;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
//---- 
   IndicatorDigits(MarketInfo(Symbol(),MODE_DIGITS)+2);
   SetIndexStyle(0,DRAW_LINE);
   if(TimeFrame == 0 || TimeFrame < Period()) TimeFrame = Period();
   SetIndexShift(0,L_Shift*TimeFrame/Period());
   draw_begin=2*L_Period*TimeFrame/Period();
//---- 
   switch(TimeFrame)
   {
   case 1     : string TF = "M1"; break;
   case 5     : TF = "M5"; break;
   case 15    : TF = "M15"; break;
   case 30    : TF = "M30"; break;
   case 60    : TF = "H1"; break;
   case 240   : TF ="H4"; break;
   case 1440  : TF="D1"; break;
   case 10080 : TF="W1"; break;
   case 43200 : TF="MN1"; break;
   default    : TF="Current";
   } 
   short_name = "AdaptiveLaguerre";
   IndicatorShortName(short_name+"("+L_Period+")"+" "+TF);
   SetIndexDrawBegin(0,draw_begin);
   SetIndexLabel(0,short_name+"("+L_Period+")"+" "+TF);
//---- 
   SetIndexBuffer(0,Laguerre);
//---- 
   return(0);
}
//+------------------------------------------------------------------+
//| AdaptiveLaguerre_v1_SW                                              |
//+------------------------------------------------------------------+
int start()
{
   int limit, y, i, shift, cnt_bars=IndicatorCounted(); 
   double price[], aPrice[], mL[], diff[], Gam[], gamma[];
    
   if(TimeFrame!=Period()) int mBars = iBars(NULL,TimeFrame); else mBars = Bars;   
   
   if(mBars != pBars)
   {
   ArrayResize(price,mBars);
   ArrayResize(aPrice,mBars);
   ArrayResize(mL,mBars);
   ArrayResize(tmp,mBars);
   ArrayResize(haClose,mBars);
   ArrayResize(haOpen,mBars);
   ArrayResize(haHigh,mBars);
   ArrayResize(haLow,mBars);
   ArrayResize(L,mBars);
   ArrayResize(diff,mBars);
   ArrayResize(Gam,mBars);
   ArrayResize(gamma,mBars);
   pBars = mBars;
   }  
   
   if(cnt_bars<1)
   {
   for(i=1;i<=draw_begin;i++) Laguerre[Bars-i]=iMA(NULL,TimeFrame,1,0,0,Price,Bars-i); 
   mcnt_bars = 0;
   }
//---- 
   if(mcnt_bars > 0) mcnt_bars--;
   
   for(y=mcnt_bars;y<mBars;y++)
   {
      if(y >= L_Period) 
      {
         if(Price <= 6) price[y] = iMA(NULL,TimeFrame,1,0,0,Price,mBars-y-1);   
         else
         if(Price > 6 && Price <= 10) price[y] = HeikenAshi(TimeFrame,Price-7,mBars-y-1);
              
         diff[y] = MathAbs(price[y] - mL[y-1]);
         
         if(GammaMode > 0)
         { 
         if(GammaMode == 1) Gam[y] = AdaptiveGamma(GammaMode,price,AdaptLength,y);
         if(GammaMode == 2) Gam[y] = AdaptiveGamma(GammaMode,diff,AdaptLength,y);
         
            switch(GammaSmoothMode)
            {
            case 0 : gamma[y] = SMA(Gam,GammaSmooth,y); break;
            case 1 : gamma[y] = EMA(Gam[y],gamma,GammaSmooth,y); break;
            case 2 : gamma[y] = Wilder(Gam,gamma,GammaSmooth,y); break;  
            case 3 : gamma[y] = LWMA(Gam,GammaSmooth,y); break;
            case 4 : gamma[y] = SineWMA(Gam,GammaSmooth,y); break;
            case 5 : gamma[y] = TriMA(Gam,GammaSmooth,y); break;
            case 6 : gamma[y] = LSMA(Gam,GammaSmooth,y); break;
            case 7 : gamma[y] = SMMA(Gam,gamma,GammaSmooth,y); break;
            case 8 : gamma[y] = HMA(Gam,GammaSmooth,y); break;
            case 9 : gamma[y] = ZeroLagEMA(Gam,gamma,GammaSmooth,y); break;
            case 10: gamma[y] = DEMA(0,Gam[y],GammaSmooth,1,y); break;
            case 11: gamma[y] = T3(Gam[y],GammaSmooth,0.7,y); break;
            case 12: gamma[y] = ITrend(Gam,gamma,GammaSmooth,y); break;
            case 13: gamma[y] = Median(Gam,GammaSmooth,y); break;
            case 14: gamma[y] = GeoMean(Gam,GammaSmooth,y); break;
            case 15: gamma[y] = REMA(Gam[y],gamma,GammaSmooth,0.5,y); break;
            case 16: gamma[y] = ILRS(Gam,GammaSmooth,y); break;
            case 17: gamma[y] = IE2(Gam,GammaSmooth,y); break;
            case 18: gamma[y] = TriMA_gen(Gam,GammaSmooth,y); break;
            }
         }
         else
         gamma[y] = Gamma;
         
         for(i=0;i<L_Period;i++)
         {
            if(y == L_Period) L[y][i] = price[y];
            else
            {
               if(i == 0) L[y][i] = (1 - gamma[y])*price[y] + gamma[y]*L[y-1][i];
               else
               L[y][i] = -gamma[y]*L[y][i-1] + L[y-1][i-1] + gamma[y]*L[y-1][i];
            aPrice[y-i] = L[y][i];
            }
         }
         mL[y] = TriMA_gen(aPrice,L_Period,y);  
                       
      if(TimeFrame == Period()) Laguerre[mBars-y-1] = mL[y];
      }
      mcnt_bars = mBars-1;
   }
   
   if(TimeFrame > Period())
   { 
      if(cnt_bars>0) cnt_bars--;
      limit = Bars-cnt_bars+TimeFrame/Period()-1;
      
      for(shift=0,y=0;shift<limit;shift++)
      {
      if (Time[shift] < iTime(NULL,TimeFrame,y)) y++; 
      Laguerre[shift] = mL[mBars-y-1];
      }
   }
   pTime=Time[0];//TimeCurrent();
//---- 
   return(0);
}
// MA_Method=0: SMA - Simple Moving Average
double SMA(double array[],int per,int bar)
{
   double Sum = 0;
   for(int i = 0;i < per;i++) Sum += array[bar-i];
     
   return(Sum/per);
}                
// MA_Method=1: EMA - Exponential Moving Average
double EMA(double price,double array[],int per,int bar)
{
   if(bar == per) double ema = price;
   else 
   if(bar > per) ema = array[bar-1] + 2.0/(1+per)*(price - array[bar-1]); 
   
   return(ema);
}
// MA_Method=2: Wilder - Wilder Exponential Moving Average
double Wilder(double array1[],double array2[],int per,int bar)
{
   if(bar == per) double wilder = SMA(array1,per,bar);
   else 
   if(bar > per) wilder = array2[bar-1] + (array1[bar] - array2[bar-1])/per; 
   
   return(wilder);
}
// MA_Method=3: LWMA - Linear Weighted Moving Average 
double LWMA(double array[],int per,int bar)
{
   double Sum = 0;
   double Weight = 0;
   
      for(int i = 0;i < per;i++)
      { 
      Weight+= (per - i);
      Sum += array[bar-i]*(per - i);
      }
   if(Weight>0) double lwma = Sum/Weight;
   else lwma = 0; 
   return(lwma);
} 
// MA_Method=4: SineWMA - Sine Weighted Moving Average
double SineWMA(double array[],int per,int bar)
{
   double pi = 3.1415926535;
   double Sum = 0;
   double Weight = 0;
  
      for(int i = 0;i < per;i++)
      { 
      Weight+= MathSin(pi*(i+1)/(per+1));
      Sum += array[bar-i]*MathSin(pi*(i+1)/(per+1)); 
      }
   if(Weight>0) double swma = Sum/Weight;
   else swma = 0; 
   return(swma);
}
// MA_Method=5: TriMA - Triangular Moving Average
double TriMA(double array[],int per,int bar)
{
   int len1 = MathCeil(per*0.5);
   int len2 = MathFloor(per*0.5);
   
   double sum = 0;
   double weight = 0;
   double sumw=0;
   for(int i = 0;i < per;i++) 
   {
   if(i<=len1-1) weight+=1;
   if(i==len1 && len1 == len2) weight = weight;
   else if(i>len1-1) weight -= 1;  
   sum += array[bar-i]*weight;
   sumw += weight; 
   } 
   double trima = sum/sumw;
   
   return(trima);
}
// MA_Method=6: LSMA - Least Square Moving Average (or EPMA, Linear Regression Line)
double LSMA(double array[],int per,int bar)
{   
   double Sum=0;
   for(int i=per; i>=1; i--) Sum += (i-(per+1)/3.0)*array[bar-per+i];
   double lsma = Sum*6/(per*(per+1));
   return(lsma);
}
// MA_Method=7: SMMA - Smoothed Moving Average
double SMMA(double array1[],double array2[],int per,int bar)
{
   if(bar == per) double smma = SMA(array1,per,bar);
   else 
   if(bar > per)
   {
   double Sum = 0;
   for(int i = 0;i < per;i++) Sum += array1[bar-i-1];
   smma = (Sum - array2[bar-1] + array1[bar])/per;
   }
   return(smma);
}                
// MA_Method=8: HMA - Hull Moving Average by Alan Hull
double HMA(double array[],int per,int bar)
{
   double tmp[];
   int len =  MathSqrt(per);
   ArrayResize(tmp,len);
   
   if(bar == per) double hma = array[bar]; 
   else
   if(bar > per)
   {
   for(int i = 0; i < len;i++) tmp[len-i-1] = 2*LWMA(array,per/2,bar-i) - LWMA(array,per,bar-i);  
   hma = LWMA(tmp,len,len-1); 
   }  

   return(hma);
}
// MA_Method=9: ZeroLagEMA - Zero-Lag Exponential Moving Average
double ZeroLagEMA(double array1[],double array2[],int per,int bar)
{
   double alfa = 2.0/(1+per); 
   int lag = 0.5*(per - 1); 
   
   if(bar == lag) double zema = array1[bar];
   else 
   if(bar > lag) zema = alfa*(2*array1[bar] - array1[bar-lag]) + (1-alfa)*array2[bar-1];
   
   return(zema);
}
// MA_Method=10: DEMA - Double Exponential Moving Average by Patrick Mulloy
double DEMA(int num,double price,int per,double v,int bar)
{
   if(bar == 2) {double dema = price; tmp[bar][num] = dema; tmp[bar][num+1] = dema;}
   else 
   if(bar > 2) 
   {
   tmp[bar][num] = tmp[bar-1][num] + 2.0/(1+per)*(price - tmp[bar-1][num]); 
   tmp[bar][num+1] = tmp[bar-1][num+1] + 2.0/(1+per)*(tmp[bar][num] - tmp[bar-1][num+1]); 
   dema = (1+v)*tmp[bar][num] - v*tmp[bar][num+1];
   }
   return(dema);
}
// MA_Method=11: T3 by T.Tillson
double T3(double price,int per,double v,int bar)
{
   if(bar == 2) 
   {
   double T3 = price; 
   for(int k=0;k<=5;k++) tmp[bar][k] = T3;
   }
   else 
   if(bar > 2) 
   {
   double dema1 = DEMA(0,price,per,v,bar); 
   double dema2 = DEMA(2,dema1,per,v,bar); 
   T3 = DEMA(4,dema2,per,v,bar);
   }
   return(T3);
}
// MA_Method=12: ITrend - Instantaneous Trendline by J.Ehlers
double ITrend(double price[],double array[],int per,int bar)
{
   double alfa = 2.0/(per+1);
   if(bar > 7)
   double it = (alfa - alfa*alfa/4)*price[bar]+ 0.5*alfa*alfa*price[bar-1]-(alfa - 0.75*alfa*alfa)*price[bar-2]+
   2*(1-alfa)*array[bar-1] - (1-alfa)*(1-alfa)*array[bar-2];
   else
   it = (price[bar] + 2*price[bar-1]+ price[bar-2])/4;
   
   return(it);
}
// MA_Method=13: Median - Moving Median
double Median(double price[],int per,int bar)
{
   double array[];
   ArrayResize(array,per);
   
   for(int i = 0; i < per;i++) array[i] = price[bar-i];
   ArraySort(array);
   
   int num = MathRound((per-1)/2); 
   if(MathMod(per,2)>0) double median = array[num]; else median = 0.5*(array[num]+array[num+1]);
   
   return(median); 
}
// MA_Method=14: GeoMean - Geometric Mean
double GeoMean(double price[],int per,int bar)
{
   double gmean = MathPow(price[bar],1.0/per); 
   for(int i = 1; i < per;i++) gmean *= MathPow(price[bar-i],1.0/per); 
   
   return(gmean);
}
// MA_Method=15: REMA - Regularized EMA by Chris Satchwell 
double REMA(double price,double array[],int per,double lambda,int bar)
{
   double alpha =  2.0/(per + 1);
   if(bar <= 3) double rema = price;
   else 
   if(bar > 3) 
   rema = (array[bar-1]*(1+2*lambda) + alpha*(price - array[bar-1]) - lambda*array[bar-2])/(1+lambda); 
   
   return(rema);
}
// MA_Method=16: ILRS - Integral of Linear Regression Slope 
double ILRS(double price[],int per,int bar)
{
   double sum = per*(per-1)*0.5;
   double sum2 = (per-1)*per*(2*per-1)/6.0;
     
   double sum1 = 0;
   double sumy = 0;
      for(int i=0;i<per;i++)
      { 
      sum1 += i*price[bar-i];
      sumy += price[bar-i];
      }
   double num1 = per*sum1 - sum*sumy;
   double num2 = sum*sum - per*sum2;
   
   if(num2 != 0) double slope = num1/num2; else slope = 0; 
   double ilrs = slope + SMA(price,per,bar);
   
   return(ilrs);
}
// MA_Method=17: IE/2 - Combination of LSMA and ILRS 
double IE2(double price[],int per,int bar)
{
   double ie = 0.5*(ILRS(price,per,bar) + LSMA(price,per,bar));
      
   return(ie); 
}
 
// MA_Method=18: TriMAgen - Triangular Moving Average Generalized by J.Ehlers
double TriMA_gen(double array[],int per,int bar)
{
   int len1 = MathFloor((per+1)*0.5);
   int len2 = MathCeil((per+1)*0.5);
   double sum=0;
   for(int i = 0;i < len2;i++) sum += SMA(array,len1,bar-i);
   double trimagen = sum/len2;
   
   return(trimagen);
}

double HeikenAshi(int tf,int price,int bar)
{ 
   if(bar == iBars(NULL,TimeFrame)- 1) 
   {
   haClose[bar] = iClose(NULL,tf,bar);
   haOpen[bar]  = iOpen(NULL,tf,bar);
   haHigh[bar]  = iHigh(NULL,tf,bar);
   haLow[bar]   = iLow(NULL,tf,bar);
   }
   else
   {
   haClose[bar] = (iOpen(NULL,tf,bar)+iHigh(NULL,tf,bar)+iLow(NULL,tf,bar)+iClose(NULL,tf,bar))/4;
   haOpen[bar]  = (haOpen[bar+1]+haClose[bar+1])/2;
   haHigh[bar]  = MathMax(iHigh(NULL,tf,bar),MathMax(haOpen[bar], haClose[bar]));
   haLow[bar]   = MathMin(iLow(NULL,tf,bar),MathMin(haOpen[bar], haClose[bar]));
   }
   
   switch(price)
   {
   case 0: return(haClose[bar]);break;
   case 1: return(haOpen[bar]);break;
   case 2: return(haHigh[bar]);break;
   case 3: return(haLow[bar]);break;
   }
}     
   
double AdaptiveGamma(int mode,double array[],int per,int bar)
{
   double sum = 0;
   double max = array[bar], min = array[bar];
   for(int i = 0;i < per;i++) 
   {
      if(mode == 1) sum += MathAbs(array[bar-i] - array[bar-i-1]); 
      else
      if(mode == 2)
      {
      max = MathMax(max,array[bar-i-1]);
      min = MathMin(min,array[bar-i-1]);
      }
   }
   
   if(mode == 1 && sum!= 0) double eff = MathAbs(array[bar] - array[bar-per])/sum;
   else 
   if(mode == 2 && max-min != 0) eff = (array[bar] - min)/(max - min);
   else
   eff = 0;
   return(1 - eff);  
}         		